﻿//Form Upload Code based on a modified version of the multipart form upload code by
//Brian Grinstead at http://www.briangrinstead.com/blog/multipart-form-post-in-c

using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.Text;
using System.Collections.Generic;

namespace BrianGrinstead
{
	public class FormUpload
	{
		private readonly Encoding encoding = Encoding.UTF8;
		WebClient client = new WebClient();
		string FormDataBoundary = "-----------------------------28947758029299";
		string ContentType;
		string PostUrl { get; set; }
		Dictionary<string, object> Parameters { get; set; }

		public delegate void FormUploadCompletedHandler(object source);
		public event FormUploadCompletedHandler FormUploadCompleted;

		public FormUpload(string postUrl, Dictionary<string, object> postParameters)
		{
			PostUrl = postUrl;
			Parameters = postParameters;

			ContentType = "multipart/form-data; boundary=" + FormDataBoundary;

			client.Encoding = encoding;
			client.Headers[HttpRequestHeader.ContentType] = ContentType;
			client.OpenWriteCompleted += new OpenWriteCompletedEventHandler(client_OpenWriteCompleted);
		}

		public void BeginMultipartFormDataPost()
		{
			client.OpenWriteAsync(new Uri(PostUrl));
		}


		void client_OpenWriteCompleted(object sender, OpenWriteCompletedEventArgs e)
		{
			byte[] formData = GetMultipartFormData(Parameters, FormDataBoundary);

			e.Result.Write(formData, 0, formData.Length);

			e.Result.Close();

			if (FormUploadCompleted != null)
			{
				FormUploadCompleted(this);
			}
		}


		private byte[] GetMultipartFormData(Dictionary<string, object> postParameters, string boundary)
		{
			Stream formDataStream = new System.IO.MemoryStream();

			foreach (var param in postParameters)
			{
				if (param.Value is FileParameter)
				{
					FileParameter fileToUpload = (FileParameter)param.Value;

					// Add just the first part of this param, since we will write the file data directly to the Stream
					string header = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"; filename=\"{2}\";\r\nContent-Type: {3}\r\n\r\n",
						boundary,
						param.Key,
						fileToUpload.FileName ?? param.Key,
						fileToUpload.ContentType ?? "application/octet-stream");

					formDataStream.Write(encoding.GetBytes(header), 0, header.Length);

					// Write the file data directly to the Stream, rather than serializing it to a string.
					formDataStream.Write(fileToUpload.File, 0, fileToUpload.File.Length);
					// Thanks to feedback from commenters, add a CRLF to allow multiple files to be uploaded
					formDataStream.Write(encoding.GetBytes("\r\n"), 0, 2);
				}
				else
				{
					string postData = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"\r\n\r\n{2}\r\n",
						boundary,
						param.Key,
						param.Value);
					formDataStream.Write(encoding.GetBytes(postData), 0, postData.Length);
				}
			}

			// Add the end of the request
			string footer = "\r\n--" + boundary + "--\r\n";
			formDataStream.Write(encoding.GetBytes(footer), 0, footer.Length);

			// Dump the Stream into a byte[]
			formDataStream.Position = 0;
			byte[] formData = new byte[formDataStream.Length];
			formDataStream.Read(formData, 0, formData.Length);
			formDataStream.Close();

			return formData;
		}

		public class FileParameter
		{
			public byte[] File { get; set; }
			public string FileName { get; set; }
			public string ContentType { get; set; }
			public FileParameter(byte[] file) : this(file, null) { }
			public FileParameter(byte[] file, string filename) : this(file, filename, null) { }
			public FileParameter(byte[] file, string filename, string contenttype)
			{
				File = file;
				FileName = filename;
				ContentType = contenttype;
			}
		}
	}
}
