﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Drawing;
using System.IO;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;

namespace EffectiveUI.MSDN.Web
{
	/// <summary>
	/// Resizes the requested image to provide an optimized mobile thumbnail to minimize bandwith to the mobile client
	/// </summary>
	public class MobileImageThumbnailHandler : IHttpHandler
	{

		public void ProcessRequest(HttpContext context)
		{
			string pictureId = string.Empty;

			try
			{
				pictureId = context.Request.Params["Image"];
			}
			catch { }

			if (string.IsNullOrEmpty(pictureId))
			{
				context.Response.StatusCode = 404;
				return;
			}

			if (!File.Exists(context.Server.MapPath("~/Content/uploaded-images/") + pictureId))
			{
				context.Response.StatusCode = 404;
				return;
			}

			context.Response.ContentType = "image/png";

			Bitmap originalImage = new Bitmap(context.Server.MapPath("~/Content/uploaded-images/") + pictureId);

			float percent = 100.00f;
			float maxSize = 80f;

			if (originalImage.Height > originalImage.Width)
			{
				percent = maxSize / (float)originalImage.Height;
			}
			else
			{
				percent = maxSize / (float)originalImage.Width;
			}
			ScaleByPercent(originalImage, percent).Save(context.Response.OutputStream, ImageFormat.Png);
		}

		public bool IsReusable
		{
			get
			{
				return false;
			}
		}

		/// <summary>
		/// Utility function to scale an image to the specified percentage of its original size
		/// </summary>
		/// <param name="Photo">The image to scale</param>
		/// <param name="Percent">The percentage of the original size to scale the photo to</param>
		/// <returns>The scaled photo</returns>
		private Image ScaleByPercent(Image Photo, float Percent)
		{
			int sourceWidth = Photo.Width;
			int sourceHeight = Photo.Height;
			int destWidth = (int)(sourceWidth * Percent);
			int destHeight = (int)(sourceHeight * Percent);

			Bitmap bmp = new Bitmap(destWidth, destHeight,
									 PixelFormat.Format24bppRgb);
			bmp.SetResolution(Photo.HorizontalResolution,
									Photo.VerticalResolution);

			Graphics g = Graphics.FromImage(bmp);
			g.InterpolationMode = InterpolationMode.HighQualityBicubic;

			g.DrawImage(Photo,
				new Rectangle(0, 0, destWidth, destHeight),
				new Rectangle(0, 0, sourceWidth, sourceHeight),
				GraphicsUnit.Pixel);

			g.Dispose();
			return bmp;
		}

	}
}