﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml.Serialization;

namespace EffectiveUI.MSDN.Web.Utility
{
	/// <summary>
	/// This class is used to load sample data into a Class.
	/// </summary>
	public class SampleDataLoader
	{
		/// <summary>
		/// Load sample data from an XML file into the specified type
		/// </summary>
		/// <typeparam name="T">The Type being populated</typeparam>
		/// <param name="fileName">The name of the XML file that contains the serialized information</param>
		/// <returns>The Type specified on the method call</returns>
		public T LoadSampleData<T>(string fileName)
		{
			FileStream file = null;
			var filePath = Utility.GetAssemblyPath() + Path.DirectorySeparatorChar + fileName;

			try
			{
				// If file exists, open it for read
				if (File.Exists(filePath))
				{
					file = File.Open(filePath, FileMode.Open, FileAccess.Read);
				}
				else
				{
					throw new FileNotFoundException("File {0} not found!", filePath);
				}

				// Create an instance of the serializer based on the Type passed in
				var serializer = new XmlSerializer(typeof(T));

				// Deserialize the object and return it
				var output = (T)serializer.Deserialize(file);
				return output;
			}
			catch
			{
				throw;
			}
			finally
			{
				try
				{
					// Close the file, if it's open
					if (file != null)
					{
						file.Close();
					}
				}
				catch
				{
					throw;
				}

			}
		}

		/// <summary>
		/// Take an Object of specified Type and serialize it to an XML file. Existing file will be deleted.
		/// </summary>
		/// <typeparam name="T">The Type being serialized</typeparam>
		/// <param name="fileName">The name of the XML file to create</param>
		/// <param name="itemToSave">The Object to serialize</param>
		public void SaveSampleData<T>(string fileName, T itemToSave)
		{

			FileStream file = null;
			var filePath = Utility.GetAssemblyPath() + Path.DirectorySeparatorChar + fileName;

			try
			{
				// Check for the existence of the file and delete it if it already exists
				if (File.Exists(filePath))
				{
					File.Delete(filePath);
				}

				// Create the output file
				file = File.Open(filePath, FileMode.Create, FileAccess.Write);

				// Serialize the Object and flush the buffer
				var serializer = new XmlSerializer(typeof(T));
				serializer.Serialize(file, itemToSave);
				file.Flush(true);
			}
			catch
			{
				throw;
			}
			finally
			{
				// Close the file, if it's open
				if (file != null)
				{
					file.Close();
				}
			}
		}

	}
}
